use strict;

package OmniDisco::Prometheus;
$OmniDisco::Prometheus::VERSION = '0.005';
=head1 NAME

OmniDisco::Prometheus - Client library to talk to Prometheus nodes

=head1 VERSION

version 0.005

=cut


use OD::Prometheus::Metric;
use OD::Prometheus::Client;

=head1 COPYRIGHT & LICENSE
 
Copyright 2018 Athanasios Douitsis, all rights reserved.
 
This program is free software; you can use it
under the terms of Artistic License 2.0 which can be found at 
http://www.perlfoundation.org/artistic_license_2_0
 
=cut

1;
